from __future__ import annotations

import os
import traceback
from functools import wraps

import attr
import requests


@attr.s
class Alerter:
    bot_token: str = attr.ib(repr=False)
    chat_id: int = attr.ib()
    base_url: str = attr.ib(init=False, repr=False)

    def __attrs_post_init__(self):
        self.base_url = f"https://api.telegram.org/bot{self.bot_token}"

    @classmethod
    def from_environment(cls) -> Alerter:
        try:
            token: str = os.environ["ALERT_BOT_TOKEN"]
        except KeyError:
            raise KeyError("ALERT_BOT_TOKEN must be set in environment variables")

        try:
            chat_id = os.environ["ALERT_CHAT_ID"]
        except KeyError:
            raise KeyError("ALERT_CHAT_ID must be set in environment variables")

        return cls(bot_token=token, chat_id=int(chat_id))

    def send_message(self, chat_id: int, *, text: str, parse_mode: str = None):
        return requests.post(
            self.base_url + "/sendMessage",
            params={"chat_id": chat_id, "text": text, "parse_mode": parse_mode},
        )

    def exception_alert(self, func):
        @wraps(func)
        def inner(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except BaseException as e:
                self.send_message(
                    self.chat_id,
                    text=f"{type(e).__name__}({e}) in {func.__name__}\n\n{traceback.format_exc()}",
                )
                raise

        return inner



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['telegram_exception_alerts']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=19.3.0,<20.0.0', 'requests>=2.23.0,<3.0.0']

setup_kwargs = {
    'name': 'telegram-exception-alerts',
    'version': '0.1.0',
    'description': '',
    'long_description': "![](https://telegram.org/img/t_logo.svg?1)\n# Telegram Exception Alerts\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n\n## Installation\n\n```bash\npip install telegram_exception_alerts\n```\nor\n```bash\npoetry add telegram_exception_alerts\n```\n\n## Usage\n\nAfter you initialize the alerter instance you can attach the decorator to any function. If it \nraises an exception information will be send to the chat specified in `chat_id` (don't forget \nthat if you want to send notification to a channel you need to prepend that `chat_id` with `-100`).\n\n### Normal initialization\n\n```python\nfrom telegram_exception_alerts import Alerter\n\nalerter = Alerter(bot_token='YOUR_BOT_TOKEN', chat_id='YOUR_CHAT_ID')\n\n@alerter.exception_alert\ndef some_func_that_can_raise_an_exception():\n    raise RuntimeError('this is an exception')\n```\n\n### Initialization from environment (recommended)\n\nYou can also initialize the alerter from environment variables. **This is the recommended way**\nbecause it will make sure you're not committing sensitive information to the repo.\n\n* `ALERT_BOT_TOKEN` - your bot token\n* `ALERT_CHAT_ID` - your chat id to receive notifications\n\n```python\nfrom telegram_exception_alerts import Alerter\n\nalerter = Alerter.from_environment()\n\n@alerter.exception_alert\ndef some_func_that_can_raise_an_exception():\n    raise RuntimeError('this is an exception')\n```\n",
    'author': 'licht1stein',
    'author_email': 'mb@blaster.ai',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/licht1stein/telegram-exception-alerts',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
